==> linuxconf-1.22.src.tar.gz: linuxconf-1.22/dialog/clist.cc <==
/* #Specification: DIALOG / FIELD_CLIST / introduction
	clist == Column list: A list of record presented in columns.
	The FIELD_CLIST type was created 6 years after the rest of this
	project. The various objects such as DIALOG_RECORDS had for a long
	time a simple way to deal with record list. They were simple to
	use. Unfortunatly, they could not scale to handle more complex dialogs.
	For example, dialog with several lists, field and buttons.

	So we created the FIELD_CLIST, which is a multi-line field operating
	under control of a normal DIALOG object.

	At this point, the DIALOG_RECORDS are still built on the same FIELD_MENU
	objects.

	Note that the GUI front-end had a Clist object for a long time. So
	both FIELD_CLIST and DIALOG_RECORDS maps to the same GUI component.
*/
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <misc.h>
#include "dialog.h"
#include <proto.h>

class FIELD_CLIST_PRIVATE{
public:
	SSTRING header;
	SSTRINGS items;
	int nbvisible;		// How many lines to display at once
	DIALOG *dia;
	int *sel;			// Will contain the line selected
	int nolist;			// Field number in this dialog of the CLIST
};



PUBLIC FIELD_CLIST::FIELD_CLIST(
	const char *_prompt,
	int nbvisible,
	DIALOG *dia,
	PRIVATE_MESSAGE &_msg,
	int &sel)
	: FIELD (_prompt)
{
	priv = new FIELD_CLIST_PRIVATE;
	priv->nbvisible = nbvisible;
	priv->dia = dia;
	priv->sel = &sel;
	priv->nolist = 0;
	msg = &_msg;
}

PUBLIC FIELD_CLIST::~FIELD_CLIST()
{
	delete priv;
}

/*
	Record the heading of the table.
	Columns are separated by tabs
*/
PUBLIC void FIELD_CLIST::setheader(const char *s)
{
	priv->header.setfrom (s);
}

/*
	Add a new record
*/
PUBLIC void FIELD_CLIST::addrecord (const char *s)
{
	priv->items.add (new SSTRING(s));
}


PUBLIC void FIELD_CLIST::addrecordf (const char *s, ...)
{
	va_list list;
	va_start (list,s);
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	va_end (list);
	addrecord (tmp);
}
PUBLIC void FIELD_CLIST::vaddrecordf (const char *s, va_list list)
{
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	addrecord (tmp);
}


/*
	Insert a new record at a specific position
*/
PUBLIC void FIELD_CLIST::insrecord (int pos, const char *s)
{
	priv->items.insert (pos,new SSTRING(s));
}


/*
	Replace the value of a record. If the record does not exist
	the value is added to the list.
*/
PUBLIC void FIELD_CLIST::setrecord (int no, const char *s)
{
	SSTRING *item = priv->items.getitem(no);
	bool differ = false;
	bool setval = false;
	if (item == NULL){
		addrecord (s);
		differ = true;
	}else if (item->cmp(s)!=0){
		item->setfrom (s);
		differ = true;
		setval = true;
	}
	if (differ){
		char tmp[200];
		const char *dianame = priv->dia->setguiname(tmp);
		if (dianame != NULL){
			char line[1000],word[1000];
			ftitle_splitline (s,line,word);
			char tmp[1000],tmp1[1000];
			int command = setval? P_Setval : P_Clist_item;
			diagui_sendcmd (command,"%s.c%d L%d %s %s\n"
				,formatpath(tmp1,dianame)
				,priv->nolist,no
				,line,diagui_quote(word,tmp));
		}else{
			priv->dia->reset_guidone();
		}
	}
}

PUBLIC void FIELD_CLIST::setrecordf (int pos, const char *s, ...)
{
	va_list list;
	va_start (list,s);
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	va_end (list);
	setrecord (pos,tmp);
}

PUBLIC void FIELD_CLIST::vsetrecordf (int pos, const char *s, va_list list)
{
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	setrecord (pos,tmp);
}

/*
	Return the number of record in the list
*/
PUBLIC int FIELD_CLIST::getnb () const
{
	return priv->items.getnb();
}
/*
	Return one item of the list
*/
PUBLIC const char *FIELD_CLIST::getline (int no) const
{
	const char *ret = NULL;
	SSTRING *s = priv->items.getitem(no);
	if (s != NULL) ret = s->get();
	return ret;
}

/*
	Remove all items after item N
*/
PUBLIC void FIELD_CLIST::remove_last (int n)
{
	for (int i=getnb()-1; i>=n; i--) remove(i);
}

/*
	Remove one item from the list
*/
PUBLIC void FIELD_CLIST::remove (int n)
{
	priv->items.remove_del(n);
	priv->dia->reset_guidone();
}

PRIVATE void FIELD_CLIST::gui_draw (int nof, SSTRINGS &tb)
{
	guisendprompt();
	priv->nolist = nof;
	char line[1000],word[1000];
	int nbhead = ftitle_splitline (priv->header.get(),line,word);
	char tmp[1000];
	diagui_sendcmd (P_Clist,"c%d %d %s %s\n",nof,nbhead
		,line,diagui_quote(word,tmp));

	int nb = priv->items.getnb();
	for (int i=0; i<nb; i++){
		const char *s = priv->items.getitem(i)->get();
		ftitle_splitline (s,line,word);
		diagui_sendcmd (P_Clist_item,"\"\" L%d %s %s\n",i
			,line,diagui_quote(word,tmp));
	}
	diagui_sendcmd (P_End,"\n");
}
PRIVATE void FIELD_CLIST::drawtxt (WINDOW *, int)
{
}
PRIVATE MENU_STATUS FIELD_CLIST::dokey(WINDOW *, int, FIELD_MSG &, bool &)
{
	return MENU_NULL;
}
PUBLIC void FIELD_CLIST::save()
{
}
PUBLIC void FIELD_CLIST::restore()
{
}
PUBLIC void FIELD_CLIST::reload(const char *, int)
{
}
PUBLIC void FIELD_CLIST::html_draw(int)
{
}
PUBLIC MENU_STATUS FIELD_CLIST::gui_get(int no, const char *field_id, const char *actionid)
{
	MENU_STATUS ret = MENU_NULL;
	if (field_id[0] == 'c' && atoi(field_id+1)==no){
		*priv->sel = atoi(actionid+1);
		ret = MENU_MESSAGE;
	}
	return ret;
}
PUBLIC char FIELD_CLIST::getidprefix()
{
	return 'c';
}
PUBLIC int FIELD_CLIST::html_validate(int)
{
	return 0;
}
PUBLIC const char * FIELD_CLIST::get_registry_value()
{
	return NULL;
}
PUBLIC void FIELD_CLIST::set_registry_value(const char *)
{
}


PUBLIC FIELD_CLIST *DIALOG::newf_clist (
	const char *prompt,
	int nbvisible,
	PRIVATE_MESSAGE &msg,
	int &sel)
{
	FIELD_CLIST *ret = new FIELD_CLIST (prompt,nbvisible,this,msg,sel);
	add (ret);
	return ret;
}

==> linuxconf-1.24.src.tar.gz: linuxconf-1.24/dialog/clist.cc <==
/* #Specification: DIALOG / FIELD_CLIST / introduction
	clist == Column list: A list of record presented in columns.
	The FIELD_CLIST type was created 6 years after the rest of this
	project. The various objects such as DIALOG_RECORDS had for a long
	time a simple way to deal with record list. They were simple to
	use. Unfortunatly, they could not scale to handle more complex dialogs.
	For example, dialog with several lists, field and buttons.

	So we created the FIELD_CLIST, which is a multi-line field operating
	under control of a normal DIALOG object.

	At this point, the DIALOG_RECORDS are still built on the same FIELD_MENU
	objects.

	Note that the GUI front-end had a Clist object for a long time. So
	both FIELD_CLIST and DIALOG_RECORDS maps to the same GUI component.
*/
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <misc.h>
#include "dialog.h"
#include <proto.h>

class FIELD_CLIST_PRIVATE{
public:
	SSTRING header;
	SSTRINGS items;
	int nbvisible;		// How many lines to display at once
	DIALOG *dia;
	int *sel;			// Will contain the line selected
	int nolist;			// Field number in this dialog of the CLIST
};



PUBLIC FIELD_CLIST::FIELD_CLIST(
	const char *_prompt,
	int nbvisible,
	DIALOG *dia,
	PRIVATE_MESSAGE &_msg,
	int &sel)
	: FIELD (_prompt)
{
	priv = new FIELD_CLIST_PRIVATE;
	priv->nbvisible = nbvisible;
	priv->dia = dia;
	priv->sel = &sel;
	priv->nolist = 0;
	msg = &_msg;
}

PUBLIC FIELD_CLIST::~FIELD_CLIST()
{
	delete priv;
}

/*
	Record the heading of the table.
	Columns are separated by tabs
*/
PUBLIC void FIELD_CLIST::setheader(const char *s)
{
	priv->header.setfrom (s);
}

/*
	Add a new record
*/
PUBLIC void FIELD_CLIST::addrecord (const char *s)
{
	priv->items.add (new SSTRING(s));
}


PUBLIC void FIELD_CLIST::addrecordf (const char *s, ...)
{
	va_list list;
	va_start (list,s);
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	va_end (list);
	addrecord (tmp);
}
PUBLIC void FIELD_CLIST::vaddrecordf (const char *s, va_list list)
{
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	addrecord (tmp);
}


/*
	Insert a new record at a specific position
*/
PUBLIC void FIELD_CLIST::insrecord (int pos, const char *s)
{
	priv->items.insert (pos,new SSTRING(s));
}


/*
	Replace the value of a record. If the record does not exist
	the value is added to the list.
*/
PUBLIC void FIELD_CLIST::setrecord (int no, const char *s)
{
	SSTRING *item = priv->items.getitem(no);
	bool differ = false;
	bool setval = false;
	if (item == NULL){
		addrecord (s);
		differ = true;
	}else if (item->cmp(s)!=0){
		item->setfrom (s);
		differ = true;
		setval = true;
	}
	if (differ){
		char tmp[200];
		const char *dianame = priv->dia->setguiname(tmp);
		if (dianame != NULL){
			char line[1000],word[1000];
			ftitle_splitline (s,line,word);
			char tmp[1000],tmp1[1000];
			int command = setval? P_Setval : P_Clist_item;
			diagui_sendcmd (command,"%s.c%d L%d %s %s\n"
				,formatpath(tmp1,dianame)
				,priv->nolist,no
				,line,diagui_quote(word,tmp));
		}else{
			priv->dia->reset_guidone();
		}
	}
}

PUBLIC void FIELD_CLIST::setrecordf (int pos, const char *s, ...)
{
	va_list list;
	va_start (list,s);
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	va_end (list);
	setrecord (pos,tmp);
}

PUBLIC void FIELD_CLIST::vsetrecordf (int pos, const char *s, va_list list)
{
	char tmp[10000];
	vsnprintf (tmp,sizeof(tmp)-1,s,list);
	setrecord (pos,tmp);
}

/*
	Return the number of record in the list
*/
PUBLIC int FIELD_CLIST::getnb () const
{
	return priv->items.getnb();
}
/*
	Return one item of the list
*/
PUBLIC const char *FIELD_CLIST::getline (int no) const
{
	const char *ret = NULL;
	SSTRING *s = priv->items.getitem(no);
	if (s != NULL) ret = s->get();
	return ret;
}

/*
	Remove all items after item N
*/
PUBLIC void FIELD_CLIST::remove_last (int n)
{
	for (int i=getnb()-1; i>=n; i--) remove(i);
}

/*
	Remove one item from the list
*/
PUBLIC void FIELD_CLIST::remove (int n)
{
	priv->items.remove_del(n);
	priv->dia->reset_guidone();
}

PRIVATE void FIELD_CLIST::gui_draw (int nof, SSTRINGS &tb)
{
	guisendprompt();
	priv->nolist = nof;
	char line[1000],word[1000];
	int nbhead = ftitle_splitline (priv->header.get(),line,word);
	char tmp[1000];
	diagui_sendcmd (P_Clist,"c%d %d %s %s $vsize=%d\n",nof,nbhead
		,line,diagui_quote(word,tmp),priv->nbvisible);

	int nb = priv->items.getnb();
	for (int i=0; i<nb; i++){
		const char *s = priv->items.getitem(i)->get();
		ftitle_splitline (s,line,word);
		diagui_sendcmd (P_Clist_item,"\"\" L%d %s %s\n",i
			,line,diagui_quote(word,tmp));
	}
	diagui_sendcmd (P_End,"\n");
}
PRIVATE void FIELD_CLIST::drawtxt (WINDOW *, int)
{
}
PRIVATE MENU_STATUS FIELD_CLIST::dokey(WINDOW *, int, FIELD_MSG &, bool &)
{
	return MENU_NULL;
}
PUBLIC void FIELD_CLIST::save()
{
}
PUBLIC void FIELD_CLIST::restore()
{
}
PUBLIC void FIELD_CLIST::reload(const char *, int)
{
}
PUBLIC void FIELD_CLIST::html_draw(int)
{
}
PUBLIC MENU_STATUS FIELD_CLIST::gui_get(int no, const char *field_id, const char *actionid)
{
	MENU_STATUS ret = MENU_NULL;
	if (field_id[0] == 'c' && atoi(field_id+1)==no){
		*priv->sel = atoi(actionid+1);
		ret = MENU_MESSAGE;
	}
	return ret;
}
PUBLIC char FIELD_CLIST::getidprefix()
{
	return 'c';
}
PUBLIC int FIELD_CLIST::html_validate(int)
{
	return 0;
}
PUBLIC const char * FIELD_CLIST::get_registry_value()
{
	return NULL;
}
PUBLIC void FIELD_CLIST::set_registry_value(const char *)
{
}


PUBLIC FIELD_CLIST *DIALOG::newf_clist (
	const char *prompt,
	int nbvisible,
	PRIVATE_MESSAGE &msg,
	int &sel)
{
	FIELD_CLIST *ret = new FIELD_CLIST (prompt,nbvisible,this,msg,sel);
	add (ret);
	return ret;
}
